using System;

namespace Geometry
{
	/// <summary>
	/// Summary description for Area.
	/// </summary>
	public class Area : IGeometry
	{
    PolylineCollection mpLines;

		public Area()
		{
      mpLines = new PolylineCollection();
    }
    public Area(Polyline border)
    {
      mpLines = new PolylineCollection();
      mpLines.Add(border);
    }
    public Area(PolylineCollection border)
    {
      mpLines = border;
    }
    public Area(Polyline[] border)
    {
      mpLines = new PolylineCollection(border);
    }
    #region IGeometry Members

    public int Dimension
    {
      get
      { return mpLines.Dimension; }
    }

    public int Topology
    {
      get
      { return 2; }
    }

    public bool Intersects(IGeometry other)
    {
      if (other.Topology > this.Topology)
      { return other.Intersects(this); }

      if (other is IMultipartGeometry)
      {
        IMultipartGeometry pMulti = (IMultipartGeometry) other;
        if (pMulti.HasSubparts)
        {
          foreach (IGeometry pSubpart in pMulti.Subparts(this.Extent))
          {
            if (this.Extent.Intersects(pSubpart.Extent) == false)
            { continue; }

            if (this.Intersects(pSubpart))
            { return true; }
          }
          return false;
        }
      }
      // other has no subparts

      int origTopo = other.Topology;
      if (other.Topology == 2)
      { 
        other = other.Border; 
      }

      if (other.Topology == 1)
      { // check lines
        IBox pBox = other.Extent;
        foreach (Polyline pLine in mpLines)
        {
          if (pLine.Extent.Intersects(pBox) == false)
          { continue; }

          if (pLine.Intersects(other))
          { return true; }
        }
        // all lines lie either completely within or outside
        if (origTopo == 2) // other polygon lies either completey within or outside
        { other = other.Border; } // TODO
      }

      if (this.Extent.Intersects(pBox) == false)
      { return false; } // should be checked already before calling this method!


      // Point
      if (other.Topology == 0)
      { return this.Intersects((IPoint) other); }

      if (other == this) // same Line (no self intersect !)
      { return false; }

      if (other.GetType() != typeof(Line))
      { throw new System.NotImplementedException(other.GetType().ToString()); }

      double u = -1;
      double v =  0;
      if (this.Intersect((Line) other,ref u,ref v) == false)
      { return false; }

      if (0 <= u && u <= 1 && 0 <= v && v <= 1)
      { return true; }

      return false;
    }

    public bool Intersects(IBox box)
    {
      throw new System.NotImplementedException();
    }

    public Box Extent
    {
      get
      { return mpLines.Extent; }
    }
    IBox IGeometry.Extent
    { get { return this.Extent; } }

    public PolylineCollection Border
    {
      get
      { return mpLines; }
    }
    IGeometry IGeometry.Border
    { get { return this.Border; } }

    public Area Project(IProjection projection)
    {
      return new Area(mpLines.Project(projection));
    }

    IGeometry IGeometry.Project(IProjection projection)
    { return this.Project(projection); }

    #endregion
  }
}
