using System;

namespace Geometry
{
  public class Box : IBox
  {
    private Point mMin;
    private Point mMax;
    private int mDimension;

    public Box(Point min,Point max)
    {
      mDimension = min.Dimension;
      mMin = min;
      mMax = max;
    }
    public Box(Point p0,Point p1,bool verify)
    {
      if (p0.Dimension < p1.Dimension)
      {mDimension = p0.Dimension;}
      else
      {mDimension = p1.Dimension;}
      if (verify) 
      {
        for (int i = 0; i < mDimension; i++) 
        {
          double t;
          if (p0[i] > p1[i]) 
          {
            t = p1[i];
            p1[i] = p0[i];
            p0[i] = t;
          }
        }
      }
      mMin = p0;
      mMax = p1;
    }
    // properties
    public Point Min
    {
      get
      {return mMin;}
    }
    IPoint IBox.Min
    { get {return this.Min; } }
    public Point Max
    {
      get
      {return mMax;}
    }
    IPoint IBox.Max
    { get {return this.Max; } }

    // methodes
    public bool IsWithin(IPoint p,int[] dimensionList)
    {
      foreach (int i in dimensionList) 
      {
        if (p[i] < mMin[i])
        { return false; }
        if (p[i] > mMax[i])
        { return false; }
      }
      return true;
    }
    public bool IsWithin(IPoint p)
    {
      return IsWithin(p,Geometry.DimensionList(mDimension));
    }
    public int ExceedDimension(IBox box,int[] dimensionList)
    {
      foreach (int i in dimensionList) 
      {
        if (box.Min[i] < mMin[i])
        {return -i - 1;}
        if (box.Max[i] > mMax[i])
        {return  i + 1;}
      }
      return 0;
    }
    public int ExceedDimension(IBox box)
    {
      return ExceedDimension(box,Geometry.DimensionList(mDimension));
    }
    public bool IsWithin(IBox box,int[] dimensionList)
    {
      return (ExceedDimension(box,dimensionList) == 0);
    }
    public bool IsWithin(IBox box)
    {
      return IsWithin(box,Geometry.DimensionList(mDimension));
    }

    public bool Intersects(IBox box,int[] dimensions)
    {
      foreach (int i in dimensions) 
      {
        if (box.Min[i] > mMax[i] || box.Max[i] < mMin[i])
        { return false;}
      }
      return true;
    }
    public bool Intersects(IBox box)
    {
      return Intersects(box,Geometry.DimensionList(mDimension));
    }
    public bool Intersects(IGeometry other)
    {
      return other.Intersects((Box) this);
    }
    public override bool Equals(object obj)
    {
      Box cmpr = obj as Box;
      if (cmpr == null) 
      { return false; }
      return mMin.Equals(cmpr.mMin) && mMax.Equals(cmpr.Max);
    }
    public override int GetHashCode()
    {
      return mMin.GetHashCode();
    }

    /// <summary>
    /// set this box to the bounding box of this and box
    /// </summary>
    /// <param name="box"></param>
    /// <param name="dimension"></param>
    public void Include(IBox box,int dimension)
    {
      for (int i = 0; i < dimension; i++)
      {
        if (box.Min[i] < mMin[i])
        { mMin[i] = box.Min[i]; }
        if (box.Max[i] > mMax[i])
        { mMax[i] = box.Max[i]; }
      }
    }

    /// <summary>
    /// set this box to the bounding box of this and box
    /// </summary>
    /// <param name="box"></param>
    public void Include(IBox box)
    {
      Include(box,Math.Min(mDimension,box.Dimension));
    }

    /// <summary>
    /// set this box to the bounding box of this and point
    /// </summary>
    /// <param name="box"></param>
    public void Include(Point point,int dimension)
    {
      for (int i = 0; i < dimension; i++)
      {
        if (point[i] < mMin[i])
        { mMin[i] = point[i]; }
        else if (point[i] > mMax[i])
        { mMax[i] = point[i]; }
      }
    }

    /// <summary>
    /// set this box to the bounding box of this and point
    /// </summary>
    /// <param name="box"></param>
    public void Include(Point point)
    {
      Include(point,Math.Min(mDimension,point.Dimension));
    }

    #region IGeometry Members

    public int Dimension
    {
      get { return mDimension; }
    }

    public int Topology
    {
      get
      { return mDimension; }
    }

    public IBox Extent 
    {
      get
      { return this; }
    }

    public Box Border
    {
      get
      { return this; }
    }
    IGeometry IGeometry.Border
    { get { return this.Border; } }

    IGeometry IGeometry.Project(IProjection projection)
    { return null; }

    #endregion

    #region ICloneable Members

    public Box Clone()
    {
      return new Box(mMin.Clone(),mMax.Clone());
    }
    IBox IBox.Clone()
    { return this.Clone(); }

    #endregion
  }
}
