using System;

namespace Geometry
{
  [Flags]
  public enum Relation { Intersect, ExtentIntersect }
  /// <summary>
  /// Summary description for Geometry.
  /// </summary>
  public class Geometry
  {
    private static int[] mDim_ = new int[] {};
    public static int[] DimensionX = new int[] {0};
    public static int[] DimensionXY = new int[] {0,1};
    public static int[] DimensionXYZ = new int[] {0,1,2};
    private static int[][] mDimList = new int[][] 
      { mDim_,DimensionX,DimensionXY,DimensionXYZ };

    [System.Diagnostics.DebuggerStepThrough()]
    public static int[] DimensionList(int dimension)
    {
      if (dimension < 3)
      {
        return mDimList[dimension];
      }
      else
      {
        int[] dim = new int[dimension];
        for (int i = 0; i < dimension; i++)
        { dim[i] = i; }
        return dim;
      }
    }

    public static double epsi = 1e-12;
    /// <summary>
    /// Cut two lines
    /// </summary>
    /// <param name="s0">start point of 1. line</param>
    /// <param name="dir0">direction of 1. line</param>
    /// <param name="s1">start point of 2. line</param>
    /// <param name="dir1">direction of 2. line</param>
    /// <param name="f">factor of 1. line or square distance between parallel lines</param>
    /// <returns>intersection point or null if directions are parallel</returns>
    public static Point2D CutDirDir(Point s0,Point dir0, Point s1,Point dir1,
      out double f)
      /*
        (x1)     (a1)     (x2)      (a2)     (x)
        (  ) + t*(  )  =  (  )  + v*(  ) ==> ( )
        (y1)     (b1)     (y2)      (b2)     (y)

        return : Point2D : lines cut each other at Point (non parallel)
                 null    : lines are parallel
                           t = square of distance between the parallels
      */
    {
      Point p = s1 - s0;
      double det;

      det = dir0.VectorProduct(dir1); // -dir0.X * dir1.Y + dir0.Y * dir1.X;
      if (Math.Abs(det) > epsi * (Math.Abs(s0.X) + Math.Abs(s1.X) + 
        Math.Abs(s0.Y) + Math.Abs(s1.Y)) / 1000.0) 
      {
        f = p.VectorProduct(dir1) / det;
        return (Point2D) (s0 + f * dir0);
      } 
      else 
      {
        // parallel lines 
        det = dir0.VectorProduct(p);
        f = (det * det) / dir0.OrigDist2(2);
        return null;
      }
    }

    public static Point2D OutCircleCenter(Point p0,Point p1,Point p2)
    {
      double f;
      Point s0 = 0.5 * (p0 + p1);
      Point dir0 = p1 - p0;
      Point s1 = 0.5 * (p1 + p2);
      Point dir1 = p2 - p1;
      return CutDirDir(s0,new Point2D(dir0.Y, -dir0.X),s1,new Point2D(dir1.Y, -dir1.X),out f) ;
    }
    public static double TriArea(Point2D p0,Point2D p1,Point2D p2)
    {
      Point2D d0 = (Point2D) (p1 - p0);
      Point2D d1 = (Point2D) (p2 - p0);
      return d0.VectorProduct(d1);
    }
  }
}
