using System;
using System.Collections;

namespace Geometry
{
  /// <summary>
  /// Summary description for IGeometry.
  /// </summary>
  public interface IBox : IGeometry
  {
    IPoint Min { get; }
    IPoint Max { get; }
    IBox Clone();
    bool IsWithin(IBox box);
    bool IsWithin(IBox box,int[] dimensionList);
    bool IsWithin(IPoint point);
    bool IsWithin(IPoint point,int[] dimensionList);
    void Include(IBox box);
  }

  public interface IGeometry
  {
    int Dimension { get; }
    int Topology { get; }
    bool Intersects(IGeometry other);
    bool Intersects(IBox box);
    IBox Extent { get; }
    IGeometry Border { get; }
    IGeometry Project(IProjection projection);
  }

  public interface IMultipartGeometry
  {
    bool HasSubparts { get; }
    IEnumerable Subparts();
    IEnumerable Subparts(IBox box);
  }

  public interface IPoint: IGeometry
  {
    double X { get; set; }
    double Y { get; set; }
    double this[int index] { get; set; }
  }

  public interface IProjection
  {
    IPoint Project(IPoint point);
  }

  public class GeometryCollection:CollectionBase
  {
    public GeometryCollection()
    { }
    public GeometryCollection(int capacity)
    {
      InnerList.Capacity = capacity;
    }
    public GeometryCollection(IGeometry[] data)
    {
      InnerList.AddRange(data);
    }

    public int Add(IGeometry value)
    {
      return List.Add(value);
    }
    public void Remove(IGeometry value)
    {
      List.Remove(value);
    }
    public bool Contains(IGeometry value)
    {
      return List.Contains(value);
    }
    public void AddRange(GeometryCollection data)
    {
      InnerList.AddRange(data);
    }
    public void Insert(int index, IGeometry value)
    {
      List.Insert(index,value);
    }
    public IGeometry this[int index]
    {
      get
      { return (IGeometry) List[index]; }
      set
      { List[index] = value; }
    }
  }
}
