﻿using System;
using System.Diagnostics;
using Grid;
using GPSLearning;
using System.IO;

namespace ANNVelocityModel
{
  public static class ANNCommitteeVelocityModel
  {
    private static readonly VeloActivationNetworkCommittee _annCommittee = (VeloActivationNetworkCommittee)VeloActivationNetworkCommittee.Load("ANNCommittee.bin");

    public static double ANNCommittee_StepCost(double h0, MapPoint mf0, double slope0_2,
      double h1, MapPoint mf1, double slope1_2, double distance)
    {


      var cmf = new ClassifiedMapPointFeatures((MapPointFeatures)mf1);

      // Return "infinite" cost for impassable areas
      const double infinity = 9999.0;
      if (cmf.Passable == Passable.Forbidden)
      {
        return infinity;
      }
      else if (cmf.Passable == Passable.ForbiddenButPathsAllowed)
      {
        if (cmf.Paths == Paths.None)
        {
          return infinity;
        }
      }

      double climb = (h1 - h0) / distance;
      double slope0 = Math.Sqrt(slope0_2);
      double slope1 = Math.Sqrt(slope1_2);
      double slope = (slope0 + slope1) / 2;
      double[] inputVector = TrainingSet.CreateInputVector(cmf, climb, slope, _annCommittee.InputVectorType, _annCommittee.NormalizeInputVector);
      double velocity = _annCommittee.Compute(inputVector) * _annCommittee.OutputScaleFactor;
      double cost = distance / velocity;

      //Debug.WriteLine("Cost: " + cost + " (Distance: " + distance + "   Velocity: " + velocity);
      return cost;
    }
  }
}
