﻿using System;
using System.Diagnostics;
using AForge.Neuro;
using Grid;
using GPSLearning;
using System.IO;

namespace ANNVelocityModel
{
  public static class ANNVelocityModel
  {
    private static readonly VeloActivationNetwork _ann = (VeloActivationNetwork)VeloActivationNetwork.Load("ANN.bin");

    public static double ANN_StepCost(double h0, MapPoint mf0, double slope0_2,
      double h1, MapPoint mf1, double slope1_2, double distance)
    {
      

      var cmf = new ClassifiedMapPointFeatures((MapPointFeatures)mf1);

      // Return "infinite" cost for impassable areas
      const double infinity = 9999.0;
      if (cmf.Passable == Passable.Forbidden)
      {
        return infinity;
      }
      else if (cmf.Passable == Passable.ForbiddenButPathsAllowed)
      {
        if (cmf.Paths == Paths.None)
        {
          return infinity;
        }
      }

      double climb = (h1 - h0) / distance;
      double slope0 = Math.Sqrt(slope0_2);
      double slope1 = Math.Sqrt(slope1_2);
      double slope = (slope0 + slope1)/2;
      double[] inputVector = TrainingSet.CreateInputVector(cmf, climb, slope, _ann.InputVectorType, _ann.NormalizeInputVector);
      double velocity = _ann.Compute(inputVector)[0] * _ann.OutputScaleFactor;
      double cost = distance/velocity;

      /*
      Debug.WriteLine("");
      Debug.WriteLine("Map: " + cmf.Forest + ", " + cmf.Water + ", " + cmf.Undergrowth + ", " + cmf.Paths);
      Debug.WriteLine("Slopes: " + climb + ", " + slope);
      Debug.WriteLine("Speed: " + velocity.ToString("N2") + "(" + (1000.0/(60*velocity)) + "min/km)");
      */
      //Debug.WriteLine("Cost: " + cost + " (Distance: " + distance + "   Velocity: " + velocity);
      return cost;
    }
  }
}
