﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Grid;

namespace GPSLearning
{
  public partial class KeyFeatureTestCreationForm : Form
  {
    private KeyFeatureTestSet _keyFeatureTestSet;

    public KeyFeatureTestCreationForm()
    {
      InitializeComponent();
      Init();
    }

    public KeyFeatureTestCreationForm(string keyFeatureTestFile)
    {
      InitializeComponent();
      Init();

      _keyFeatureTestSet = TrainingSetSerializer.DeserializeKeyFeatureTestSetFromXml(keyFeatureTestFile);
      lbxTests.DataSource = _keyFeatureTestSet.Tests;
      ((CurrencyManager)lbxTests.BindingContext[_keyFeatureTestSet.Tests]).Refresh(); // Refreshes contents of lbxTests
    }

    public void Init()
    {
      _keyFeatureTestSet = new KeyFeatureTestSet();
      cmbForest.DataSource = Enum.GetValues(typeof(Forest));
      cmbWater.DataSource = Enum.GetValues(typeof(Water));
      cmbUndergrowth.DataSource = Enum.GetValues(typeof(Undergrowth));
      cmbPaths.DataSource = Enum.GetValues(typeof(Paths));
      cmbGround.DataSource = Enum.GetValues(typeof(Ground));
      lbxTests.DataSource = _keyFeatureTestSet.Tests;

      // Ignore ground for now
      cmbGround.SelectedItem = Ground.None;
      cmbGround.Enabled = false;
      lblGround.Enabled = false;
    }

    private void btnAddInstance_Click(object sender, EventArgs e)
    {
      var forest = (Forest) cmbForest.SelectedItem;
      var water = (Water) cmbWater.SelectedItem;
      var undergrowth = (Undergrowth) cmbUndergrowth.SelectedItem;
      var paths = (Paths) cmbPaths.SelectedItem;
      var ground = (Ground) cmbGround.SelectedItem;
      
      var cmf = new ClassifiedMapPointFeatures(forest, water, undergrowth, paths, ground, Passable.Passable);
      double climb = double.Parse(txtClimb.Text);
      double slope = double.Parse(txtSlope.Text);
      var instance = new TrainingInstance(climb, slope, cmf);
      _keyFeatureTestSet.Add(instance);
      ((CurrencyManager)lbxTests.BindingContext[_keyFeatureTestSet.Tests]).Refresh(); // Refreshes contents of lbxTests
    }

    private void btnSaveKeyFeatureTest_Click(object sender, EventArgs e)
    {
      dlgSave.Filter = "*.xml|*.xml";
      if (dlgSave.ShowDialog(this) == DialogResult.OK)
      {
        TrainingSetSerializer.SerializeKeyFeatureTestSetToXml(dlgSave.FileName, _keyFeatureTestSet);
      }
    }

    private void btnRemove_Click(object sender, EventArgs e)
    {
      foreach (TrainingInstance t in lbxTests.SelectedItems)
      {
        _keyFeatureTestSet.Remove(t);
      }
      ((CurrencyManager)lbxTests.BindingContext[_keyFeatureTestSet.Tests]).Refresh(); // Refreshes contents of lbxTests
    }

    private void btnMoveUp_Click(object sender, EventArgs e)
    {
      int nofSelectedItems = lbxTests.SelectedIndices.Count;
      if (nofSelectedItems > 1)
      {
        int newIndex = lbxTests.SelectedIndices[nofSelectedItems - 1];
        lbxTests.SelectedIndices.Clear();
        lbxTests.SelectedIndex = newIndex;
        return;
      }
      if (nofSelectedItems == 1)
      {
        int index = lbxTests.SelectedIndex;
        TrainingInstance t = _keyFeatureTestSet.Tests[index];
        TrainingInstance other = _keyFeatureTestSet.Tests[index - 1];
        _keyFeatureTestSet.Tests[index - 1] = t;
        _keyFeatureTestSet.Tests[index] = other;
        ((CurrencyManager)lbxTests.BindingContext[_keyFeatureTestSet.Tests]).Refresh(); // Refreshes contents of lbxTests
        lbxTests.SelectedIndices.Clear();
        lbxTests.SelectedIndex = index - 1;
      }
    }

    private void btnMoveDown_Click(object sender, EventArgs e)
    {
      int nofSelectedItems = lbxTests.SelectedIndices.Count;
      if (nofSelectedItems > 1)
      {
        int newIndex = lbxTests.SelectedIndices[nofSelectedItems - 1];
        lbxTests.SelectedIndices.Clear();
        lbxTests.SelectedIndex = newIndex;
        return;
      }
      if (nofSelectedItems == 1)
      {
        int index = lbxTests.SelectedIndex;
        TrainingInstance t = _keyFeatureTestSet.Tests[index];
        TrainingInstance other = _keyFeatureTestSet.Tests[index + 1];
        _keyFeatureTestSet.Tests[index + 1] = t;
        _keyFeatureTestSet.Tests[index] = other;
        ((CurrencyManager)lbxTests.BindingContext[_keyFeatureTestSet.Tests]).Refresh(); // Refreshes contents of lbxTests
        lbxTests.SelectedIndices.Clear();
        lbxTests.SelectedIndex = index + 1;
      }
    }
  }
}
