using System;

namespace Grid
{
  public interface ILockable
  {
    void LockBits();
    void UnlockBits();
  }

  public abstract class BaseGrid
  {
    private GridExtent _extent;
    private Type _type;

    public BaseGrid(GridExtent extent, Type type)
    {
      _extent = extent;
      _type = type;
    }
    public object Value(double x, double y)
    {
      int ix, iy;
      _extent.GetNearest(x, y, out ix, out iy);
      return this[ix, iy]; // this[ix, iy];
    }

    public T Offset<T>(int ix, int iy) where T : BaseGrid
    { return null; }
    public object this[int ix, int iy]
    {
      get { return GetThis(ix, iy); }
      set { SetThis(ix, iy, value); }
    }
    protected abstract object GetThis(int ix, int iy);
    protected abstract void SetThis(int ix, int iy, object value);

    public Type Type
    { get { return _type; } }

    public GridExtent Extent
    {
      get { return _extent; }
    }

    public int Topology
    {
      get
      { return 2; }
    }
  }

  public abstract class BaseGrid<T> : BaseGrid
  {
    public BaseGrid(GridExtent extent)
      : base(extent, typeof(T))
    { }
    public new virtual T Value(double x, double y)
    {
      return (T)base.Value(x, y);
    }
    public new abstract T this[int ix, int iy] { get; set; }
    protected sealed override object GetThis(int ix, int iy)
    { return this[ix, iy]; }
    protected sealed override void SetThis(int ix, int iy, object value)
    { this[ix, iy] = (T)value; }
  }
}
