
using System;
using System.Reflection;
using Grid;

namespace LeastCostPathUI
{
	/// <summary>
	/// Summary description for Common.
	/// </summary>
	public class Common
	{
    public static double CostPath_StepCost(double h0, MapPoint m0, double slope0_2,
      double h1, MapPoint m1, double slope1_2, double distance)
    {
      // Using velocity grid
      if (m0 is MapPointVelocity && m1 is MapPointVelocity)
      {
        MapPointVelocity v0 = (MapPointVelocity)m0;
        MapPointVelocity v1 = (MapPointVelocity)m1;
        double v = (v0.Velocity + v1.Velocity) / 2.0;
        double dhPos = h1 - h0;
        if (dhPos < 0)
        { dhPos = 0; }

        double cost;
        cost = distance / v + dhPos * 7;

        if (v < 0.99)
        { // not road
          double slope2 = (slope0_2 + slope1_2) / 2;
          cost += slope2 * 7.5 * distance;
        }
        if (double.IsNaN(cost))
        { return distance * 100; }
        return cost;
      }
      // Using map feature grids
      else if (m0 is MapPointFeatures && m1 is MapPointFeatures)
      {
        ClassifiedMapPointFeatures cm0 = new ClassifiedMapPointFeatures((MapPointFeatures)m0);
        ClassifiedMapPointFeatures cm1 = new ClassifiedMapPointFeatures((MapPointFeatures)m1);
        double v0 = ComputeFlatTerrainVelocity(cm0);
        double v1 = ComputeFlatTerrainVelocity(cm1);
        
        double v = (v0 + v1)/2.0;
        double dhPos = h1 - h0;
        if (dhPos < 0)
        { dhPos = 0; }

        double cost;
        cost = distance / v + dhPos * 7;

        if (v < 0.99)
        { // not road
          double slope2 = (slope0_2 + slope1_2) / 2;
          cost += slope2 * 7.5 * distance;
        }
        if (double.IsNaN(cost))
        //{ return distance * 100; }
        { return 9999.0; }
        return cost;
      }
      else
      {
        throw new NotImplementedException();
      }

    }

    public static double ComputeFlatTerrainVelocity(ClassifiedMapPointFeatures cm)
    {
      if (cm.Passable == Passable.Forbidden)
      {
        return 0.01;
      }
      if (cm.Passable == Passable.ForbiddenButPathsAllowed && cm.Paths == Paths.None)
      {
        return 0.0;
      }
      if (cm.Paths != Paths.None)
      {
        return 1.0;
      }
      /*if (cm.Paths == Paths.Road || cm.Paths == Paths.VehicleTrack || cm.Paths == Paths.Footpath)
      {
        return 1.0;
      }
      if (cm.Paths == Paths.SmallPath || cm.Paths == Paths.LessDistinctSmallPath || cm.Paths == Paths.NarrowRide)
      {
        return 0.9;
      }*/
      if (cm.Forest == Forest.SlowRunning || cm.Undergrowth == Undergrowth.SlowRunning)
      {
        return 0.7;
      }
      if (cm.Forest == Forest.DifficultToRun || cm.Undergrowth == Undergrowth.DifficultToRun)
      {
        return 0.4;
      }
      if (cm.Forest  == Forest.VeryDifficultToRun)
      {
        return 0.15;
      }
      return 0.9;
    }

    public static int InitColors(byte[] r,byte[] g,byte[] b)
    { 
      if (r.Length == 256)
      {
        int i,k;
        for (i = 0; i <= 42; i++) 
        {
          k = i;
          r[i] = 255;
          g[i] = (byte) (6 * k);
          b[i] = 0;
        }
        for (i = 43; i <= 85; i++) 
        {
          k = (byte) (i - 43);
          r[i] = (byte) (255 - 6 * k);
          g[i] = 255;
          b[i] = 0;
        }

        for (i = 86; i <= 128; i++) 
        {
          k = (byte) (i - 86);
          g[i] = 255;
          b[i] = (byte) (6 * k);
          r[i] = 0;
        }
        for (i = 129; i <= 170; i++) 
        {
          k = (byte) (i - 129);
          g[i] = (byte) (255 - 6 * k);
          b[i] = 255;
          r[i] = 0;
        }
  
        for (i = 171; i <= 212; i++) 
        {
          k = (byte) (i - 171);
          b[i] = 255;
          r[i] = (byte) (6 * k);
          g[i] = 0;
        }
        for (i = 213; i <= 255; i++) 
        {
          k = (byte) (i - 213);
          b[i] = (byte) (255 - 6 * k);
          r[i] = 255;
          g[i] = 0;
        }
      }
      else if (r.Length == 60)
      {
        int i, k;
        for (i = 0; i <= 9; i++)
        {
          k = i;
          r[i] = 255;
          g[i] = (byte)(25 * k);
          b[i] = 0;
        }
        for (i = 10; i <= 19; i++)
        {
          k = (byte)(i - 10);
          r[i] = (byte)(255 - 25 * k);
          g[i] = 255;
          b[i] = 0;
        }
        for (i = 20; i <= 29; i++)
        {
          k = (byte)(i - 20);
          g[i] = 255;
          b[i] = (byte)(25 * k);
          r[i] = 0;
        }
        for (i = 30; i <= 39; i++)
        {
          k = (byte)(i - 30);
          g[i] = (byte)(255 - 25 * k);
          b[i] = 255;
          r[i] = 0;
        }
        for (i = 40; i <= 49; i++)
        {
          k = (byte)(i - 40);
          b[i] = 255;
          r[i] = (byte)(25 * k);
          g[i] = 0;
        }
        for (i = 50; i <= 59; i++)
        {
          k = (byte)(i - 50);
          b[i] = (byte)(255 - 25 * k);
          r[i] = 255;
          g[i] = 0;
        }
      }
      else if (r.Length == 62)
      {
        int i, k;
        for (i = 2; i <= 11; i++)
        {
          k = (byte) (i - 2);
          r[i] = 255;
          g[i] = (byte)(25 * k);
          b[i] = 0;
        }
        for (i = 12; i <= 21; i++)
        {
          k = (byte)(i - 12);
          r[i] = (byte)(255 - 25 * k);
          g[i] = 255;
          b[i] = 0;
        }
        for (i = 22; i <= 31; i++)
        {
          k = (byte)(i - 22);
          g[i] = 255;
          b[i] = (byte)(25 * k);
          r[i] = 0;
        }
        for (i = 32; i <= 41; i++)
        {
          k = (byte)(i - 32);
          g[i] = (byte)(255 - 25 * k);
          b[i] = 255;
          r[i] = 0;
        }
        for (i = 42; i <= 51; i++)
        {
          k = (byte)(i - 42);
          b[i] = 255;
          r[i] = (byte)(25 * k);
          g[i] = 0;
        }
        for (i = 52; i <= 61; i++)
        {
          k = (byte)(i - 52);
          b[i] = (byte)(255 - 25 * k);
          r[i] = 255;
          g[i] = 0;
        }
      }
      return 0;
    }

    public static string GetMethodName(MethodInfo methodInfo)
	  {
      return string.Format("{0}.{1}", methodInfo.ReflectedType.Name, methodInfo.Name);
	  }
	}
}
