using System;
using System.Data;

namespace Mathe.Data
{
  public class SchemaTable : DataTable
  {
    public class Row : DataRow
    {
      internal Row(DataRowBuilder builder)
        : base(builder)
      { }

      public string ColumnName
      {
        get { return ColumnNameColumn.GetValue(this); }
        set { ColumnNameColumn.SetValue(this, value); }
      }

      public int ColumnOrdinal
      {
        get { return ColumnOrdinalColumn.GetValue(this); }
        set { ColumnOrdinalColumn.SetValue(this, value); }
      }
      public Type DataType
      {
        get { return DataTypeColumn.GetValue(this); }
        set { DataTypeColumn.SetValue(this, value); }
      }
    }

    public static readonly TypedColumn<string> ColumnNameColumn = new TypedColumn<string>(DbBaseReader.ColColumnName);
    public static readonly TypedColumn<int> ColumnOrdinalColumn = new TypedColumn<int>(DbBaseReader.ColColumnOrdinal);
    public static readonly TypedColumn<Type> DataTypeColumn = new TypedColumn<Type>(DbBaseReader.ColDataType);

    public SchemaTable()
      : base("SchemaTable")
    {
      Columns.Add(ColumnNameColumn.CreateColumn());
      Columns.Add(ColumnOrdinalColumn.CreateColumn());
      Columns.Add(DataTypeColumn.CreateColumn());
    }

    public new Row NewRow()
    {
      return (Row)base.NewRow();
    }

    protected override DataRow NewRowFromBuilder(DataRowBuilder builder)
    {
      Row row = new Row(builder);
      return row;
    }

    public DataRow AddSchemaColumn(string shape, Type type)
    {
      Row schemaRow = NewRow();
      schemaRow.ColumnName = shape;
      schemaRow.ColumnOrdinal = Rows.Count;
      schemaRow.DataType = type;
      Rows.Add(schemaRow);

      schemaRow.AcceptChanges();
      return schemaRow;
    }

    public void AddRow(Row row)
    {
      Rows.Add(row);
    }
  }
}