using System;
using System.Collections.Generic;

namespace Mathe.Geom
{
  public class ParamGeometryRelation
  {
    private class Info
    {
      public readonly IParamGeometry Geometry;
      public readonly IPoint Param;
      private LinkedList<IGeometry> _parents;
      private LinkedListNode<IGeometry> _parent;

      public Info(IParamGeometry geometry, IPoint param)
      {
        Geometry = geometry;
        Param = param;
      }

      public IGeometry Current
      {
        get
        {
          if (_parent != null)
          { return _parent.Value; }
          else
          { return Geometry; }
        }
      }

      public bool TryAddParent(IGeometry parent, IGeometry child)
      {
        bool result;
        if (child == Geometry || Geometry.EqualGeometry(child))
        {
          if (_parents != null)
          { throw new InvalidOperationException("Parent already added"); }
          _parents = new LinkedList<IGeometry>();
          _parents.AddFirst(parent);

          result = true;
        }
        else if (_parents != null && _parents.Last.Value == child)
        {
          _parents.AddLast(parent);
          result = true;
        }
        else
        { result = false; }

        return result;
      }

      public bool TryGetChildRelation(IGeometry parent,
        ParamGeometryRelation rel, Info asY, out ParamGeometryRelation result)
      {
        result = null;
        Info resultInfo = null;

        if (_parent != null && parent.EqualGeometry(_parent.Value))
        {
          resultInfo = new Info(Geometry, Param);
          resultInfo._parent = _parent.Previous;
        }
        else if (_parent == null && _parents != null && parent.EqualGeometry(_parents.Last.Value))
        {
          resultInfo = new Info(Geometry, Param);
          resultInfo._parent = _parents.Last.Previous;
        }
        else if (_parent == null && parent.EqualGeometry(Geometry))
        { }
        else
        {
          return false;
        }
        if (resultInfo != null)
        {
          result = new ParamGeometryRelation();

          result._isWithin = rel._isWithin;
          result._intersect = rel._intersect;
          result._offset = rel._offset;
          result._offset2 = rel._offset2;

          resultInfo._parents = _parents;

          result._X = resultInfo;
          result._Y = asY;
        }
        else
        { result = null; }

        return true;
      }
    }

    private Info _X;
    private Info _Y;
    private IPoint _offset;

    private IGeometry _intersect;
    private bool? _isWithin;
    double _offset2 = -1;

    private ParamGeometryRelation()
    { }

    public ParamGeometryRelation(IParamGeometry x, IParamGeometry y,
      IGeometry intersection)
    {
      _X = new Info(x, null);
      _Y = new Info(y, null);
      Intersection = intersection;
    }

    public ParamGeometryRelation(IParamGeometry x, IPoint xParam,
      IParamGeometry y, IPoint yParam, IPoint offset)
    {
      _X = new Info(x, xParam);
      _Y = new Info(y, yParam);
      _offset = offset;
    }

    public void AddParent(IGeometry parent, IGeometry child)
    {
      if (_X.TryAddParent(parent, child))
      { }
      else if (_Y.TryAddParent(parent, child))
      { }
      else
      { throw new InvalidOperationException("child not valid"); }
    }

    public ParamGeometryRelation GetChildRelation(IGeometry parent)
    {
      ParamGeometryRelation relation;
      if (_X.TryGetChildRelation(parent, this, _Y, out relation))
      { }
      else if (_Y.TryGetChildRelation(parent, this, _X, out relation))
      { }
      else
      { throw new InvalidOperationException("child not valid"); }
      return relation;
    }

    public IPoint XParam
    {
      get { return _X.Param; }
    }

    public IGeometry CurrentX
    {
      get { return _X.Current; }
    }

    public IGeometry Intersection
    {
      get
      {
        if (IsWithin)
        { return IntersectionAnywhere; }
        else
        { return null; }
      }
      set
      {
        _intersect = value;
        if (value != null)
        {
          _isWithin = true;
          _offset2 = 0;
        }
        else
        {
          _isWithin = false;
          _offset2 = 1;
        }
      }
    }

    public IGeometry IntersectionAnywhere
    {
      get
      {
        if (_intersect == null)
        {
          if (Offset2 == 0 && _X.Param != null)
          { _intersect = _X.Geometry.PointAt(_X.Param); }
        }
        return _intersect;
      }
    }

    public bool IsWithin
    {
      get
      {
        if (_isWithin == null)
        { _isWithin = CheckRange(_X.Param) && CheckRange(_Y.Param); }
        return _isWithin.Value;
      }
    }

    private bool CheckRange(IPoint p)
    {
      if (p == null)
      { return false; }

      int dim = p.Dimension;
      for (int i = 0; i < dim; i++)
      {
        double x = p[i];
        if (x < 0 || x > 1)
        { return false; }
      }
      return true;
    }

    public double Offset2
    {
      get
      {
        if (_offset2 < 0)
        {
          _offset2 = PointOperator.OrigDist2(_offset);
          if (_offset2 < 1.0e-12)
          { _offset2 = 0; }
        }
        return _offset2;
      }
    }
  }
}
