using System.Data;
using Mathe.Data;
using OCourse.Ext;

namespace OCourse.Route
{
  public class CombinationTable : DataTable
  {
    public class Row : DataRow
    {
      internal Row(DataRowBuilder builder)
        : base(builder)
      { }

      public string Name
      {
        get { return NameColumn.GetValue(this); }
        set { NameColumn.SetValue(this, value); }
      }

      public double DirectLKm
      {
        get { return DirectLKmColumn.GetValue(this); }
        set { DirectLKmColumn.SetValue(this, value); }
      }

      public double DirectKm
      {
        get { return DirectKmColumn.GetValue(this); }
        set { DirectKmColumn.SetValue(this, value); }
      }

      public double Climb
      {
        get { return ClimbColumn.GetValue(this); }
        set { ClimbColumn.SetValue(this, value); }
      }

      public double OptimalLKm
      {
        get { return OptimalLKmColumn.GetValue(this); }
        set { OptimalLKmColumn.SetValue(this, value); }
      }

      public double OptimalKm
      {
        get { return OptimalKmColumn.GetValue(this); }
        set { OptimalKmColumn.SetValue(this, value); }
      }

      public double OptimalCost
      {
        get { return OptimalCostColumn.GetValue(this); }
        set { OptimalCostColumn.SetValue(this, value); }
      }

      public SectionList Combination
      {
        get { return CombinationColumn.GetValue(this); }
        set { CombinationColumn.SetValue(this, value); }
      }

      public new CombinationTable Table
      {
        get { return (CombinationTable)base.Table; }
      }

    }

    public static readonly TypedColumn<string> NameColumn = new TypedColumn<string>("Name");
    public static readonly TypedColumn<double> DirectLKmColumn = new TypedColumn<double>("DirectLKm");
    public static readonly TypedColumn<double> DirectKmColumn = new TypedColumn<double>("DirectKm");
    public static readonly TypedColumn<double> ClimbColumn = new TypedColumn<double>("Climb");
    public static readonly TypedColumn<double> OptimalLKmColumn = new TypedColumn<double>("OptimalLKm");
    public static readonly TypedColumn<double> OptimalKmColumn = new TypedColumn<double>("OptimalKm");
    public static readonly TypedColumn<double> OptimalCostColumn = new TypedColumn<double>("OptimalCost");
    public static readonly TypedColumn<SectionList> CombinationColumn = new TypedColumn<SectionList>("Combination");

    public CombinationTable()
      : base("Combination")
    {
      Columns.Add(NameColumn.CreateColumn());
      Columns.Add(DirectLKmColumn.CreateColumn());
      Columns.Add(DirectKmColumn.CreateColumn());
      Columns.Add(ClimbColumn.CreateColumn());
      Columns.Add(OptimalLKmColumn.CreateColumn());
      Columns.Add(OptimalKmColumn.CreateColumn());
      Columns.Add(OptimalCostColumn.CreateColumn());
      Columns.Add(CombinationColumn.CreateColumn());
    }

    public new Row NewRow()
    {
      return (Row)base.NewRow();
    }

    protected override DataRow NewRowFromBuilder(DataRowBuilder builder)
    {
      Row row = new Row(builder);
      return row;
    }

    public void AddRow(Row row)
    {
      Rows.Add(row);
    }

    public Row AddRow(string Name, double DirectLKm, double DirectKm, double Climb,
      double OptimalLKm, double OptimalKm, double OptimalCost, SectionList Combination)
    {
      Row row = NewRow();
      row.Name = Name;
      row.DirectLKm = DirectLKm;
      row.DirectKm = DirectKm;
      row.Climb = Climb;
      row.OptimalLKm = OptimalLKm;
      row.OptimalKm = OptimalKm;
      row.OptimalCost = OptimalCost;
      row.Combination = Combination;
      Rows.Add(row);
      return row;
    }
  }
}
