using System;
using Ocad;

namespace OCourse.Route
{
  public abstract class CostInfo
  {
    public abstract double Direct { get; }
    public abstract double OptimalLength { get; }
    public abstract double OptimalCost { get; }
    public abstract double Climb { get; }
    public abstract void Add(CostInfo part);

    public double Cost()
    {
      return Direct + 10 * Climb;
    }
  }
  public class SectionInfo : CostInfo
  {
    private Control _from;
    private Control _to;

    private double _direct;
    private double _optimalLength;
    private double _optimalCost;
    private double _climb;

    private string _name;

    private SectionInfo()
    { }

    public SectionInfo(Control start, Control finish,
      double direct, double climb, double optimalLength, double optimalCost)
    {
      _from = start;
      _to = finish;

      _direct = direct;
      _climb = climb;

      _optimalLength = optimalLength;
      _optimalCost = optimalCost;
    }
    public SectionInfo(CostInfo info)
    {
      _direct = info.Direct;
      _climb = info.Climb;

      _optimalLength = info.OptimalLength;
      _optimalCost = info.OptimalCost;
    }

    public Control From
    { get { return _from; } }
    public Control To
    { get { return _to; } }

    public override double Direct
    { get { return _direct; } }

    public override double OptimalCost
    { get { return _optimalCost; } }

    public override double OptimalLength
    { get { return _optimalLength; } }

    public override double Climb
    { get { return _climb; } }

    public string Name
    {
      get { return _name; }
      set { _name = value; }
    }

    public override void Add(CostInfo part)
    {
      if (part is SectionInfo)
      {
        _direct += part.Direct;
        _climb += part.Climb;
        _optimalLength += part.OptimalLength;
        _optimalCost += part.OptimalCost;
      }
      else
      { throw new ArgumentException("Invalid part type " + part.GetType()); }
    }

    public static SectionInfo operator +(SectionInfo s0, SectionInfo s1)
    {
      SectionInfo sum = new SectionInfo();
      sum._direct = s0.Direct + s1.Direct;
      sum._climb = s0.Climb + s1.Climb;

      sum._optimalLength = s0.OptimalLength + s1.OptimalLength;
      sum._optimalCost = s0.OptimalCost + s1.OptimalCost;

      return sum;
    }

    public static SectionInfo operator *(double f, SectionInfo section)
    {
      SectionInfo prod = new SectionInfo();
      prod._direct = f * section.Direct;
      prod._climb = f * section.Climb;

      prod._optimalLength = f * section.OptimalLength;
      prod._optimalCost = f * section.OptimalCost;

      return prod;
    }
  }
}
