
using System;
using System.Collections.Generic;
using Mathe.Geom;
using Ocad;

namespace OCourse
{
  public static class Utils
  {
    public static List<string> GetCourseList(string courseFile)
    {
      List<string> courseList = new List<string>();
      using (OcadReader courseReader = OcadReader.Open(courseFile))
      {
        IList<StringParamIndex> idxList = courseReader.ReadStringParamIndices();

        foreach (StringParamIndex index in idxList)
        {
          if (index.Type == StringType.Course)
          {
            string course = courseReader.ReadCourseName(index);
            courseList.Add(course);
          }
        }
      }
      return courseList;
    }


    public static double GetLength(Control from, Control to, Setup setup)
    {
      double length = 0;
      IGeometry toGeom = to.Element.Geometry;
      if (setup != null)
      { toGeom = toGeom.Project(setup.Map2Prj); }

      Polyline line = toGeom as Polyline;
      if (line != null)
      {
        length += line.Length();
      }
      if (from == null)
      {
        return length;
      }

      IGeometry fromGeom = from.Element.Geometry;
      if (setup != null)
      { fromGeom = fromGeom.Project(setup.Map2Prj); }

      IPoint fromPoint = fromGeom as IPoint;
      if (fromPoint == null)
      {
        fromPoint = ((Polyline)fromGeom).Points.Last.Value;
      }

      IPoint toPoint;
      if (line != null)
      {
        toPoint = line.Points.First.Value;
      }
      else
      {
        toPoint = (IPoint)toGeom;
      }

      length += Math.Sqrt(Point2D.Dist2(fromPoint, toPoint));
      return length;
    }

  }
}
