
using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using Mathe.Data;
using Mathe.Geom;

namespace Ocad.Data
{
  public class OcadDbReader : DbBaseReader
  {
    #region nested classes
    private class Enumerator : IEnumerator<Element>
    {
      private readonly OcadDbReader _reader;
      private OcadReader.ElementEnumerator _enumerator;
      private Element _currentElem;
      IGeometry _intersect = null;

      public Enumerator(OcadDbReader reader, OcadInfo ocad)
      {
        _reader = reader;
        IBox extent = null;

        foreach (DbBaseParameter parameter in reader._command.Parameters)
        {
          if (parameter.Value is IGeometry)
          {
            _intersect = (IGeometry)parameter.Value;
            extent = _intersect.Extent;
          }
        }

        _enumerator = new OcadReader.ElementEnumerator(ocad, extent, true);
      }

      #region IEnumerator Members

      public void Dispose()
      { }

      public void Reset()
      {
        _enumerator.Reset();
        _currentElem = null;
      }

      object IEnumerator.Current
      { get { return Current; } }
      public Element Current
      {
        get { return _currentElem; }
      }

      public bool MoveNext()
      {
        bool value = _enumerator.MoveNext();
        bool conditions = false;
        _currentElem = null;
        while (value && conditions == false)
        {
          _currentElem = _enumerator.Current;

          conditions = _reader._command.Validate(_reader);

          if (conditions == false)
          { value = _enumerator.MoveNext(); }
        }
        return value;
      }

      #endregion
    }
    #endregion

    private OcadCommand _command;

    private Enumerator _enum;
    private int _nRows;

    public OcadDbReader(OcadCommand command, CommandBehavior behavior)
      : base(command, behavior)
    {
      _command = command;
    }

    protected override void Dispose(bool disposing)
    {
      if (_enum != null)
      {
        _enum.Dispose();
        _enum = null;
      }
      base.Dispose(disposing);
    }

    public override IEnumerator GetEnumerator()
    {
      if (_enum != null)
      { _enum.Dispose(); }

      _enum = new Enumerator(this, OcadInfo);
      _nRows = 0;
      return _enum;
    }

    public override object GetValue(int ordinal)
    {
      string name = GetName(ordinal);
      if (OcadConnection.FieldShape.Equals(name, StringComparison.InvariantCultureIgnoreCase))
      {
        return _enum.Current.Geometry;
      }
      else if (OcadConnection.FieldSymbol.Equals(name, StringComparison.InvariantCultureIgnoreCase))
      {
        return _enum.Current.Symbol;
      }
      else if (OcadConnection.FieldAngle.Equals(name, StringComparison.InvariantCultureIgnoreCase))
      {
        return _enum.Current.Angle;
      }
      else
      {
        throw new InvalidOperationException("Invalid ordinal " + ordinal);
      }
    }

    private OcadInfo _ocadInfo;
    private OcadInfo OcadInfo
    {
      get
      {
        if (_ocadInfo == null)
        {
          _ocadInfo = _command.Connection.GetReader(_command.TableName);
        }
        return _ocadInfo;
      }
    }

    public override bool Read()
    {
      if (_enum == null)
      { GetEnumerator(); }

      bool next = _enum.MoveNext();
      if (next == false)
      { }
      else
      { _nRows++; }
      return next;
    }

    public override bool NextResult()
    {
      return true;
    }

    public override int RecordsAffected
    {
      get { throw new Exception("The method or operation is not implemented."); }
    }

    protected override SchemaTable GetSchemaTableCore()
    {
      SchemaTable schema = _command.GetSchemaTable();
      return schema;
    }
  }
}
