
using System;
using System.Collections.Generic;
using System.Text;

namespace Ocad.StringParams
{
  public abstract class MultiParam
  {
    IList<string> _paraList;
    public MultiParam(string para)
    {
      _paraList = new List<string>(para.Split('\t'));
    }

    protected MultiParam()
    {
      _paraList = new List<string>();
    }

    protected IList<string> ParaList
    {
      get { return _paraList; }
    }

    public string StringParam
    {
      get
      {
        StringBuilder s = new StringBuilder(_paraList[0]);
        int n = _paraList.Count;
        for (int i = 1; i < n; i++)
        {
          s.AppendFormat("\t{0}", _paraList[i]);
        }
        s.Append("\t\t");
        return s.ToString();
      }
    }
    public string Name
    {
      get { return _paraList[0]; }
      set { _paraList[0] = value; }
    }

    protected void Add(char key, object value)
    {
      string par = string.Format("{0}{1}", key, value);
      _paraList.Add(par);
    }

    private int Index(char t)
    {
      int n = _paraList.Count;
      for (int i = 1; i < n; i++) // 0 Index = Course Name!
      {
        if (_paraList[i][0] == t)
        { return i; }
      }
      return -1;
    }

    protected string GetString(char key)
    {
      int index = Index(key);

      if (index < 0 || index > ParaList.Count)
      { return null; }

      string para = ParaList[index];

      if (para.Length < 2)
      { return null; }
      return para.Substring(1);
    }

    protected int? GetInt(char key)
    {
      string s = GetString(key);
      if (string.IsNullOrEmpty(s))
      { return null; }
      int i;
      if (int.TryParse(s, out i) == false)
      { return null; }
      return i;
    }

    protected void SetParam(char key, object value)
    {
      int idx = Index(key);
      if (key < 0)
      {
        if (value == null)
        { return; }

        _paraList.Add(key.ToString());
        idx = Index(key);
      }
      if (value == null)
      {
        _paraList.RemoveAt(idx);
      }
      _paraList[idx] = string.Format("{0}{1}", _paraList[idx][0], value);
    }
  }
}
