using System;

namespace Ocad
{
  /// <summary>
  /// Summary description for Template.
  /// </summary>
  public class Template : StringPar
  {
    public Template()
      : base(true)
    { }

    public Template(string stringParam)
      : base(stringParam, true)
    {
      StringParam = stringParam;
    }

    public double Omega
    {
      get
      { return Convert.ToDouble(_params['a']); }
      set
      { SetParam('a', value); }
    }

    public double Phi
    {
      get
      { return Convert.ToDouble(_params['b']); }
      set
      { SetParam('b', value); }
    }

    public double Dim
    {
      get
      { return Convert.ToDouble(_params['d']); }
      set
      { SetParam('d', value); }
    }

    public bool Transparent
    {
      get
      { return Convert.ToBoolean(_params['t']); }
      set
      { SetParam('t', value ? 1 : 0); }
    }
    public double X0
    {
      get
      { return Convert.ToDouble(_params['x']); }
      set
      { SetParam('x', value); }
    }
    public double Y0
    {
      get
      { return Convert.ToDouble(_params['y']); }
      set
      { SetParam('y', value); }
    }
    public double DX
    {
      get
      { return Convert.ToDouble(_params['u']); }
      set
      { SetParam('u', value); }
    }
    public double DY
    {
      get
      { return Convert.ToDouble(_params['v']); }
      set
      { SetParam('v', value); }
    }
    public bool Visible
    {
      get
      { return Convert.ToBoolean(_params['r']); }
      set
      {
        SetParam('r', value ? 1 : 0);
        SetParam('s', value ? 1 : 0);
      }
    }

    public void Georeference(double dxx, double dxy, double dyx, double dyy,
      double xMin, double yMax, int nx, int ny, Setup setup)
    {
      double df = setup.Scale / 1000.0;
      DX = dxx / df;
      DY = -dyy / df;
      X0 = (xMin + nx / 2.0 * dxx - setup.PrjTrans.X) / df;
      Y0 = (yMax + ny / 2.0 * dyy - setup.PrjTrans.Y) / df;
    }
  }
}
