
using System;
using System.Collections.Generic;
using Grid;
using Mathe.Geom;
using Mathe.Geom.Projection;
using Ocad;

namespace Tracking
{
  public class Track
  {
    private Course _course;
    private IList<TrackPoint> _track;
    
    public Track(Course course, IList<TrackPoint> track)
    {
      _course = course;
      _track = track;
    }

    public Track(IList<TrackPoint> track)
    {
      _course = null;
      _track = track;
    }

    public void Fit()
    {
      double lon0 = 0;
      if (_track.Count > 0)
      {
        lon0 = (int)(_track[0].Long / 6) + 3;
      }
      Projection prj = new Utm();
      List<IPoint> utm = new List<IPoint>();
      foreach (TrackPoint tp in _track)
      {
        Point gg = new Point2D((tp.Long - lon0) * Math.PI / 180.0,
          tp.Lat * Math.PI / 180.0);

        IPoint p = prj.Gg2Prj(gg);
        utm.Add(p);
      }

      IPoint center = GetCenter(utm);
    }

    /// <summary>
    /// Converts the lat, long corrdinates of the track into UTM.
    /// </summary>
    private static List<IPoint> LatLongToUtm(List<TrackPoint> trackPoints)
    {
      /*double lon0 = ((int)(trackPoints[0].Long/6))*6 + 3;
      Projection prj = new Utm();*/
      var utmPntList = new List<IPoint>();
      foreach (TrackPoint tp in trackPoints)
      {
        /*Point gg = new Point2D((tp.Long - lon0)*Math.PI/180.0,
                               tp.Lat*Math.PI/180.0);
        IPoint p = prj.Gg2Prj(gg);*/
        double utmNorthing, utmEasting;
        string zone;
        LatLonToUtmConverter.LatLongtoUtm(tp.Lat, tp.Long, out utmNorthing, out utmEasting, out zone);
        IPoint p = new Point2D(utmEasting, utmNorthing);
        utmPntList.Add(p);
      }
      return utmPntList;
    }

    public static Polyline GpxToPolyline(string gpxFile)
    {
      var pntList = (List<TrackPoint>)GpxIO.ReadData(gpxFile);
      List<IPoint> utmPntList = LatLongToUtm(pntList);
      return new Polyline(utmPntList);
    }

    private IPoint GetCenter(IList<IPoint> line)
    {
      Point center = new Point2D();
      IPoint p1 = null;
      double sumW = 0;
      foreach (IPoint point in line)
      {
        IPoint p0 = p1;
        p1 = point;

        if (p0 == null)
        { continue; }
        double w = Math.Sqrt(Point2D.Dist2(p0, p1));

        center = center + w / 2.0 * (Point.CastOrCreate(p0) + Point.CastOrCreate(p1));
        sumW += w;
      }

      center = (1 / sumW) * center;
      return center;
    }

    /// <summary>
    /// Uses the supplied height grid to assign the height value of each track point in
    /// the supplied list of track points.
    /// </summary>
    /// <param name="trackPoints"></param>
    /// <param name="heightGrid"></param>
    public static void AssignHeight(IList<TrackPoint> trackPoints, DoubleGrid heightGrid)
    {
      foreach (TrackPoint pnt in trackPoints)
      {
        pnt.H = heightGrid.Value(pnt.X, pnt.Y, EGridInterpolation.bilinear);
      }
    }


  }
}
