using System;
using System.Collections;

namespace Geometry
{
  /// <summary>
  /// Summary description for Area.
  /// </summary>
  public class PolylineCollection:CollectionBase, IGeometry
  {
    Box mExtent;

    private void UpdateExtent(Box extent)
    {
      if (mExtent == null)
      { mExtent = extent; }
      else
      { mExtent.Include(extent); }
    }

    private void Init()
    {
      foreach (Polyline line in List)
      {
        UpdateExtent(line.Extent);
      }
    }

    public PolylineCollection()
    {
      Init();
    }

    public PolylineCollection(Polyline[] lineList)
    {
      InnerList.AddRange(lineList);
      Init();
    }
    public PolylineCollection(Polyline line)
    {
      List.Add(line);
      Init();
    }

    public Polyline this[int index]
    {
      get 
      {return (Polyline) List[index];}
      set
      {List[index] = value;}
    }

    public int Add(Polyline value)
    {
      int i = List.Add(value);
      UpdateExtent(value.Extent);
      return i;
    }

    public void Insert(int index, Polyline value)
    {
      List.Insert(index,value);
      UpdateExtent(value.Extent);
    }

    public void Remove(Polyline value)
    {
      List.Remove(value);
      // TODO: update Extent
    }

    public bool Contains(Polyline value)
    {
      return List.Contains(value);
    }

    public int IndexOf(Polyline value)
    {
      return List.IndexOf(value);
    }

    #region IGeometry Members

    public int Dimension
    {
      get
      { return mExtent.Dimension; }
    }

    public int Topology
    {
      get
      { return 1; }
    }

    public bool Intersects(IBox box)
    {
      Box pBox = Extent.Clone();
      if (box.IsWithin(pBox))
      { return true; }
      if (box.Intersects(pBox) == false)
      { return false; }
      // TODO : calculate
      return false;
    }
    public bool Intersects(IGeometry other)
    {
      if (other.Topology > this.Topology)
      { return other.Intersects(this); }
      return false; // TODO
    }

    public Box Extent
    {
      get
      { return mExtent.Clone(); }
    }
    IBox IGeometry.Extent
    { get { return this.Extent; } }

    public PointCollection Border
    {
      get // TODO get
      { throw new System.NotImplementedException("TODO"); }
    }
    IGeometry IGeometry.Border
    { get { return this.Border; } }

    public PolylineCollection Project(IProjection projection)
    {
      PolylineCollection pPrj = new PolylineCollection();
      foreach (Polyline polyline in List)
      { pPrj.Add(polyline.Project(projection)); }

      return pPrj;
    }
    IGeometry IGeometry.Project(IProjection projection)
    { return Project(projection); }


    #endregion

    public int PointCount
    {
      get
      {
        int n = 0;
        foreach (Polyline polyline in List)
        { n += polyline.Points.Count; }
        return n;
      }
    }
  }
}
