using System;
using System.Collections;

namespace Geometry
{
  /// <summary>
  /// Summary description for SegmentList.
  /// </summary>
  public class SegmentList : IEnumerable
  {
    #region nested classes
    private class MyEnumerator : IEnumerator
    {
      private Polyline mpPolyline;
      private int miPos;
      private Line mpLine;

      public MyEnumerator(Polyline polyline)
      {
        mpPolyline = polyline;
        Reset();
      }
      #region IEnumerator Members

      public void Reset()
      {
        miPos = 0;
      }

      public object Current
      {
        get
        { return mpLine; }
      }

      public bool MoveNext()
      {
        if (miPos >= mpPolyline.Segments.Count)
        { return false; }

        if (mpPolyline.SegmentParts == null || mpPolyline.SegmentParts[miPos] == null)
        { mpLine = new Line(mpPolyline.Points[miPos],mpPolyline.Points[miPos + 1]); }
        else
        { 
          mpLine = mpPolyline.SegmentParts[miPos].Complete(mpPolyline.Points[miPos],
            mpPolyline.Points[miPos + 1]);
        }

        miPos ++;
        return true;
      }

      #endregion

    }

    #endregion nested classes

    private Polyline mpPolyline;

    public SegmentList(Polyline polyline)
    {
      mpPolyline = polyline;
    }

    public Line this[int index]
    {
      get
      {
        if (mpPolyline.SegmentParts == null || mpPolyline.SegmentParts[index] == null)
        { return new Line(mpPolyline.Points[index],mpPolyline.Points[index + 1]); }
        else
        { 
          return mpPolyline.SegmentParts[index].Complete(mpPolyline.Points[index],
            mpPolyline.Points[index + 1]);
        }
      }
    }

    #region ICollection Members

    public int Count
    {
      get
      { return mpPolyline.Points.Count - 1; }
    }

    #endregion

    #region IEnumerable Members

    public IEnumerator GetEnumerator()
    {
      return new MyEnumerator(mpPolyline);
    }

    #endregion
  }
}
